#include "mazeclass.as"

// Grid Maze - by Jim Bumgardner
//

MW = 12;	// initial width and height
MH = 12;

function GridMaze(w,h)
{
	this.w = w;
	this.h = h;
	this.digMethod = 2;	// use 1 for "snakier"
	this.idleReport = spinDice;
	this.completionFunc = FinishMaze;
}

GridMaze.prototype = new Maze();

// These are the functions we need to define to make a maze

// Determine the number of edges in each cell.  For a grid-type maze
// this is constant (4 for squares, 6 for hexagons, etc.).  See PolarMaze 
// for an example where it varies, based on cell location.
//
GridMaze.prototype.getNbrEdges = function(addr)
{
	return 4;
}

// The parent class treats the maze as a one-dimensional list of cells.
// This function tells the parent class how long to make the array.
GridMaze.prototype.getMaxCells = function()
{
	return this.w * this.h;
}

// This is the function that determines the topology of the maze.
// Given a cell address (from 0 to n-1) it figures out which other cells
// are attached to this one, and by which edges.
//
// We return a list of neighbor cells, indexed by connecting edge.
// 
GridMaze.prototype.findNeighbors = function(cellAddr)
{
	// trace("Getting neighbors for " + cellAddr);
	var vec = new CellList(4);	// could also use getNbrEdges(), but don't need it here
	var x = cellAddr % this.w;
	var y = Math.floor(cellAddr / this.w);
	if (x < this.w-1)
		vec.cells[0] = this.cells[cellAddr + 1];
	if (y > 0)
		vec.cells[1] = this.cells[cellAddr - this.w];
	if (x > 0)
		vec.cells[2] = this.cells[cellAddr - 1];
	if (y < this.h-1)
		vec.cells[3] = this.cells[cellAddr + this.w];
	return vec;
}

// This draws a maze cell
//
GridMaze.prototype.drawCell = function(mc, cell)
{
	var x = cell.address % this.w;
	var y = Math.floor( cell.address / this.w );
	// trace("Drawing " + x + " " + y);
	if (cell.edgeFlags & 1) {
		mc.drawLine(mc.LM + (x+1) * mc.TW, mc.TM + (y+1) * mc.TH,
								mc.LM + (x+1) * mc.TW, mc.TM + (y) * mc.TH);
	}
	if (cell.edgeFlags & 2) {
		mc.drawLine(mc.LM + (x+1) * mc.TW, mc.TM+(y)*mc.TH, 
							  mc.LM+(x)*mc.TW, mc.TM+(y)*mc.TH);
	}
	if (cell.edgeFlags & 4) {
		mc.drawLine(mc.LM+(x)*mc.TW, mc.TM+(y)*mc.TH, 
								mc.LM+(x)*mc.TW, mc.TM+(y+1)*mc.TH);
	}
	if (cell.edgeFlags & 8) {
		mc.drawLine(mc.LM+(x)*mc.TW, mc.TM+(y+1)*mc.TH, 
								mc.LM+(x+1)*mc.TW, mc.TM+(y+1)*mc.TH);
	}
}

// The remaining functions are optional, but common...
//
MovieClip.prototype.drawLine = function(x1,y1,x2,y2)
{
		this.moveTo(x1, y1);
		this.lineTo(x2, y2);
}

function drawMyMaze()
{
	mc.TW = 300/(this.maze.w + 2);	// tile sizes
	mc.TH = 300/(this.maze.h + 2);
	mc.TM = mc.TW;	// margins
	mc.LM = mc.TH;


	// trace("Draw My Maze");
	mc.clear();
	mc.lineStyle(2, 0x000000, 100);
	for (var i = this.maze.cells.length; --i >= 0;)
	{
		this.maze.drawCell(this, this.maze.cells[i]);
	}
	this.onEnterFrame = undefined;
}

// Dice handling functions (the dice button generates a new maze)
//

function spinDice(r)
{
	dice_mc._rotation = r*360;	
}


function animButtonGrow()
{
	delta = this.tScale - this._xscale;
	if (delta >= 1) 
	{
		this._xscale += delta/2;
		this._yscale += delta/2;
	}
	else {
		this._xscale = this.tScale;
		this._yscale = this.tScale;
		this.onEnterFrame = undefined;
	}
}

function growButton()
{
	this.tScale = 125;
	this.active = true;
	this.onEnterFrame = animButtonGrow;
}

function shrinkButton()
{
	this.tScale = 100;
	this.active = false;
	this.onEnterFrame = animButtonGrow;
}

function FinishMaze()
{
	mc.onEnterFrame = drawMyMaze;
}

function resetGridMaze()
{
	if (this.active)
		mc.maze.init();
}

function biggerGridMaze()
{
	if (this.active) {
		if (mc.maze.w < 32) {
			mc.maze.w += 2;
			mc.maze.h += 2;
		}
		mc.maze.init();
	}
}

function smallerGridMaze()
{
	if (this.active) {
		if (mc.maze.w > 2) {
			mc.maze.w -= 2;
			mc.maze.h -= 2;
		}
		mc.maze.init();
	}
}

function init()
{
	// Basic Maze Initialization
	this.mc = this.createEmptyMovieClip("maze_mc", 0);
	this.mc._x = 0;
	this.mc._y = 0;
	this.mc.maze = new GridMaze(MW,MH);
	this.mc.maze.init();

	// this.mc.gotoAndStop(1);
	this.dice_mc.onMouseUp = resetGridMaze;
	this.dice_mc.onRollOver = growButton;
	this.dice_mc.onRollOut =  shrinkButton;

	this.big_mc.onMouseUp = biggerGridMaze;
	this.big_mc.onRollOver = growButton;
	this.big_mc.onRollOut =  shrinkButton;

	this.small_mc.onMouseUp = smallerGridMaze;
	this.small_mc.onRollOver = growButton;
	this.small_mc.onRollOut =  shrinkButton;

}

init();
